% The demo is about the Automatic Relevance Determination of Adaptive Variational Bayes Sparse 
% Decomposition for Micro-Cracks Detection in Thermal Sensing
% If you use this code please cite this paper
% Peng Lu, Bin Gao, W.L. Woo, X.Q. Li, G.Y. Tian Automatic Relevance Determination of Adaptive
% Variational Bayes Sparse Decomposition for Micro-Cracks Detection in Thermal Sensing 
% Copyright 2017  Peng Lu, Bin Gao, W.L. Woo, X.Q. Li, G.Y. Tian
% Report bugs to Bin Gao
% bin_gao -at- uestc.edu.cn
% Checked 28/6/2017

clc;
close all;
clear all;

% Loading the inductive thermography data
% e.g. in order to save space, we only take the first 31 frames for this demo
load data; 

[F,N,T] = size(data);
bian = 20; % Setting the size of the thermal patterns sub-patch
c = bian^2;
a = F/bian;
b = N/bian;

for i=1:F
    for j=1:N
        data_dif(i,j,:)=diff(data(i,j,:));
    end
end

% data conversion
A = zeros(c,a*b,T-1);
for i = 1:T-1
    for j = 1:b
        for k = 1:a
            A(:,k+a*(j-1),i) = reshape(data_dif((k-1)*bian+1:(k-1)*bian+bian,(j-1)*bian+1:(j-1)*bian+bian,i),c,1);
        end
    end
end

for i=1:T-1
    data_temp=A(:,:,i);
    Y(:,i)=data_temp(:);
end

% the label of the sub-patch
BB1 = [1:1:a*b];

[COEFF, SCORE] = pca(Y);
Y = Y(:,:) * COEFF(:,1:1);

options.verbose = 1;
options.initial_rank = 'auto'; % This sets to the maximum possible rank
% options.initial_rank = 300; % or we can use a value. 
options.inf_flag = 1; % inference flag is equal to 1.
options.MAXITER = 500;
%Estimate noise variance? (beta is inverse noise variance)
options.UPDATE_BETA = 1; 
options.mode = 'VB';
fprintf('Running VBRPCA...\n')
X0 = Y;
kk=0.8;

for ii=1:1000

[X_hat, A_hat, B_hat, E_hat, BB] = ARDVB(X0,options,BB1,kk,bian);

[a1,b1,c1] = kmeans(E_hat(:,1),2);

% calculate the thershold
BBB = b1(1)/b1(2);

% stop the iteration, for different situation, 1000 or 0.001 can be changed.
if BBB > 1000 || BBB < 0.001
    break;
end

% adaptive update the parameter, if the abs(sum(E_hat)) is very large or
% small, you can set use the another function: kk = kk + 0.1;
 kk = kk + 0.012*abs(sum(E_hat)); 

end

[re, ce] = size(BB);

% reconstructed image
tu1 = ones(F,N);
tu = min(E_hat(:,1)) .* tu1;
for k = 1:ce
    f = fix(BB(1,k)/a);
    ff = rem(BB(1,k),a);
    if ff ~= 0
    tu((ff-1)*bian+1:ff*bian,f*bian+1:(f+1)*bian) = reshape(E_hat((k-1)*bian^2+1:k*bian^2,1),bian,bian);
    else
    tu((a-1)*bian+1:a*bian,(f-1)*bian+1:f*bian) = reshape(E_hat((k-1)*bian^2+1:k*bian^2,1),bian,bian); 
    end
end

%show the image
figure
imagesc(data(:,:,4));
set(gca,'fontsize',18);
xlabel('Spatial x axis','fontsize',18)
ylabel('Spatial y axis','fontsize',18)
title('Original thermal image at frame 4','fontsize',18);
axis xy
figure
imagesc(tu);
set(gca,'fontsize',18);
xlabel('Spatial x axis','fontsize',18)
ylabel('Spatial y axis','fontsize',18)
title('Thermal Patterns','fontsize',18);
axis xy